#ifndef __SPI_FLASH_H
#define __SPI_FLASH_H

#include <stdint.h>
#include <stdbool.h>

#define MX_PIN_MOSI         20
#define MX_PIN_MISO         21
#define MX_PIN_SCK          19
#define MX_PIN_CS           17

#define CMD_PROGRAM                 0x02
#define CMD_READ                    0x03
#define CMD_RDSR                    0x05
#define CMD_RDCR                    0x15
#define CMD_WREN                    0x06
#define CMD_REMS                    0x90
#define CMD_REMS_ADDRESS_DEFAULT    0x00
#define CMD_SECTOR_ERASE            0x20
#define CMD_CHIP_ERASE              0x60

#define REG_SR_BIT_WIP              0
#define REG_SR_BIT_WEL              1
#define REG_SR_BIT_BP0              2
#define REG_SR_BIT_BP1              3
#define REG_SR_BIT_BP2              4
#define REG_SR_BIT_BP3              5
#define REG_SR_BIT_QE               6
#define REG_SR_BIT_SRWD             7


uint32_t mx_init(void);

void mx_read_rems(uint8_t * manufacturer_id, uint8_t * device_id);

uint8_t mx_read_status_register(void);

uint8_t mx_read_config_register(void);

void mx_read(uint32_t address, uint8_t * data_ptr, uint32_t data_length);

void mx_write_enable(void);

void mx_program(uint32_t address, uint8_t * data_ptr, uint32_t data_length);

void mx_sector_erase(uint32_t address);

void mx_chip_erase(void);

#endif
